############################################################################################
# 
# 
# This script demonstrates R programming techniques sampling the attributes of a 
# vector-format polygon map using a regularly-spaced point grid. 
# The script produces a ESRI-format point shapefile containing the sampling results. 
#
# Date: July 21, 2008
# For: NCEAS Scientific Programming Web Site Examples
# Author: Rick Reeves, Scientific Programmer, NCEAS
# Copyright 2008 National Center for Ecological Analysis and Synthesis
############################################################################################
TheDriver <- function()
{
   library(sp)
   library(maptools)
   library(PBSmapping)
#
# set the working directory for your local machine...
#   
   setwd("e:/Projects/JeremyEcophylogenetics/UseCase/SampleVectorPolygonsRasterGrid/Download")
   debug(BaileysTest)
   BaileysTest()
   print(sprintf("End of example!\n"))
}
#
# R script demonstrates overlay of regularly-spaced point grid
# at specified spatial resolution over a polygon-based data layer,
# AND the sampling of the polygon layer at each point location. 
# Uses the R sp() package methods to perform the sampling, and 
# some methods from PBSMapping to plot the data.
#
BaileysTest <- function()
{
   browser()
# 
# Test dataset: Continental US states with 50 attached demographic attributes
# This produces a SpatialPolygonsDataFrame.
#
   EcoRegion = readShapePoly("BaileysEcoRegionsUSA")
#
   NewData = EcoRegion@data[,3:5]
   EcoRegionTrunc = EcoRegion
   EcoRegionTrunc@data = NewData
#                                                       
# Calculate the sampling grid parameters, get the boundaries from the new SPDF's 
# bounding box element. Make the grid a bit bigger, span the entire shape file.
# Then, create a GridTopology object containing the parameters.
#
   vals = EcoRegionTrunc@bbox
   deltaLong = as.integer((vals[1,2] - vals[1,1]) + 1.5)
   deltaLat  = as.integer((vals[2,2] - vals[2,1]) + 1.5)
#
# grid resolution in fractional degress.
#
   gridRes = 1.0
#   
   gridSizeX = deltaLong / gridRes
   gridSizeY = deltaLat / gridRes
#
# GridTopology object forms the basis for the sampling grid        
#
   GridTopoOneDeg = GridTopology(vals[,1],c(gridRes,gridRes), c(gridSizeX,gridSizeY))
#
# Create the SpatialGrid object....
#
   SamplingGridOneDeg = SpatialGrid(GridTopoOneDeg)
#
# for educational purposes, we could compare the two bounding boxes....
#
#  EcoRegionTrunc@bbox
#  SamplingGrid@bbox       
#                                                                      
# We get a more suitable output from the overlay() method    
# if we use a SpatialPoint object as an input, so promote
# the SpatialGrid object to one. 
#
  SamplingPointsOneDeg = as(SamplingGridOneDeg,"SpatialPoints")
  NumPoints = length(SamplingPointsOneDeg@coords[,1])
  OneDegEvents <- as.EventData(data.frame(EID=1:NumPoints,X=SamplingPointsOneDeg@coords[,1],
                               Y=SamplingPointsOneDeg@coords[,2]),projection = "NA")
#
# Have a look at the sampling grid: plot polygons and then overplot the points                                                               
#
   EcoRegionPS = (SpatialPolygons2PolySet(EcoRegionTrunc))
   plotPolys(EcoRegionPS)
   addPoints(OneDegEvents,col="red",pch=20)     
#
# Generate and plot the two-degree sampling grid            
#
# grid resolution in fractional degress.
#
   gridRes = 2.0
#   
   gridSizeX = deltaLong / gridRes
   gridSizeY = deltaLat / gridRes
#
# GridTopology object forms the basis for the sampling grid        
#
   GridTopoTwoDeg = GridTopology(vals[,1],c(gridRes,gridRes), c(gridSizeX,gridSizeY))
#
# Create the SpatialGrid object....
#
   SamplingGridTwoDeg = SpatialGrid(GridTopoTwoDeg)        
   SamplingPointsTwoDeg = as(SamplingGridTwoDeg,"SpatialPoints")
#
# Define a new graphics device, generate the plot
#
   dev.set(1)   
   plotPolys(EcoRegionPS, xlab = "W Longitude", ylab = "N Latitude")
   NumPoints = length(SamplingPointsTwoDeg@coords[,1])
   TwoDegEvents <- as.EventData(data.frame(EID=1:NumPoints,X=SamplingPointsTwoDeg@coords[,1],
                               Y=SamplingPointsTwoDeg@coords[,2]),projection = "NA")   
# 
# plot the two-degree grid by itself   
#
   dev.set(1)
   plotPoints(TwoDegEvents, col = "green", pch = 20)                                  
# 
# plot points against map background
#
   dev.set(1)   
   plotPolys(EcoRegionPS, xlab = "W Longitude", ylab = "N Latitude")   
   addPoints(TwoDegEvents,col="green",pch = 20)
#
# We promoted the SpatialGrid to SpatialPoints to get the proper case for 'overlay': 
# The sampling output type depends on the order of the parameters: see 'overlay-methods' sp help page
#
# SamplingResultsDF is a data frame with one entry per sampling point (with sampled attribute values attached)
# ol2 is a vector of integers, each entry is the polygon ID beneath one sampling point. 
#
# Use the overlay() method to extract a polygon attribute value 
# at each point location within the polygon data set boundary
#
   SamplingResultsDFOneDeg = overlay(EcoRegionTrunc,SamplingPointsOneDeg)
   SamplingResultsDFTwoDeg = overlay(EcoRegionTrunc,SamplingPointsTwoDeg)
   
#   ol2 = overlay(SamplingPoints,EcoRegionTrunc)                                                                                                             
#
# Lets create a point shape file to contain the 
# sampling results. The attribute table will include
# all three Bailey's measures along with the corresponding
# point location. 
# Make some modifications to the overlay results data frame 
# so that the shape file has usable attributes. 
#
# First, add the sampling point coordinates
# 
   Temp1 = cbind(SamplingPointsOneDeg@coords,SamplingResultsDFOneDeg)
   Temp2 = cbind(SamplingPointsTwoDeg@coords,SamplingResultsDFTwoDeg)
#
# Next, convert the ECOCODE attribute from a factor to a character string
# 
   ecoCode = as.character(Temp1$ECOCODE)             
   flags = is.na(ecoCode)
   ecoCode[flags] = "NoMatch"
   OutputDataFrame = cbind(Temp1[1:4],ecoCode)
#
# Finally, generate a point shape file that includes the EcoRegion attribute.
#   
   write.pointShape(SamplingPointsOneDeg@coords,OutputDataFrame,"BaileyEcoRegionGridOneDegJuly21")
#
# For the Two-Degree grid, ALSO create a 'clipped' dataset with ONLY points inside USA.
# 
   ecoCode = as.character( Temp2$ECOCODE)
   flags = is.na(ecoCode)
   ecoCode[flags] = "NoMatch"
   OutputDataFrame = cbind(Temp2[1:4],ecoCode)
   write.pointShape(SamplingPointsTwoDeg@coords,OutputDataFrame,"BaileyEcoRegionGridTwoDegJuly21")
#
#   Create subset of the point grid consisting only of those within the USA boundaries
#
   TwoDegEventsWithCodes = cbind(TwoDegEvents,ecoCode)
   PointsInsideRegions = findPolys(TwoDegEventsWithCodes,EcoRegionPS)
   PointIndexesInsideRegions = sort(unique(PointsInsideRegions$EID))
   PointsInsideRegions = TwoDegEventsWithCodes[PointIndexesInsideRegions,]
#
# Add them to the plot
#   
   addPoints(PointsInsideRegions,col="green",pch = 20)   
#
# Finally, Write the clipped point shapefile, read it back in, and and plot it.
#   
   write.pointShape(cbind(PointsInsideRegions$X,PointsInsideRegions$Y),PointsInsideRegions,"BaileyEcoRegionGridTwoDegClip")
#
   print("hit key to process The FINAL plot..")
   browser()
   TwoDegInUSA = readShapePoints("BaileyEcoRegionGridTwoDegClip")
#
# look at the structure of the point data object 
# before and after conversion to Events
#
   str(TwoDegInUSA)
   NumPoints = length(TwoDegInUSA@data[,1])
   PointEvents <- as.EventData(data.frame(EID=1:NumPoints ,X=TwoDegInUSA@coords[,1],
                                        Y=TwoDegInUSA@coords[,2]))# ,projection = "NA")
   str(PointEvents)
#   
   plotPolys(EcoRegionPS, xlab = "W Longitude", ylab = "N Latitude")   
   addPoints(PointEvents,col="orange",pch = 20)
#
# End of example!
#    
}
